searchData={"items":[{"type":"module","title":"Beethoven","doc":"A Decentralized failover and peer-to-peer node finding framework for Elixir.\nUsing a TCP socket, Elixir nodes running Beethoven can find each other within a pre-defined network range.\n\n# Public API\n\n- `ready?/0` -> Returns true if Beethoven has fully initialized. False if not.\n    Fully initialized is determined when the RoleMgmt Assigns server.\n\n- `until_ready/1` -> Holds the thread until Beethoven has fully initialized, or provided timeout value has become exhausted.\n    Default timeout value is `5_000` milliseconds.\n\n- `get_node_status/0` -> Returns the status of the node within the Beethoven cluster.\n    Possible returns are: `:standalone` & `:clustered`.\n\n- `get_cluster_nodes/0` -> Returns a list of all the nodes have have joined the Beethoven cluster.\n\n- `get_active_cluster_nodes/0` -> Similar to `get_cluster_nodes/0`, but only provides nodes that are active/online.\n\n- `get_detailed_cluster_nodes/0` -> Detailed list of all the nodes that have joined the Beethoven cluster + associated metadata.\n\n- `get_roles/0` -> Returns all Beethoven hosted roles with their associated metadata.\n\n- `whois_hosting/0` -> Looks up, and returns a list of nodes that host a given Beethoven role.\n\n- `allocate/0` -> Wrapper for `Beethoven.Allocator.allocate/0`.\n\n---\n\n# Architecture\n\nBeethoven is made of multiple smaller services that provide a depth of failover and loadbalancing functionality.\n\n## `CoreServer`\n\nMonitors nodes in the Mnesia cluster.\nThe server provides a behaviour that allows for PIDs to request a callback in the event of a node status change.\nThe callback is called `alert_me/0`. This function is available to any module using `CoreServer`.\nOnce used, the caller will be sent a GenServer cast containing the updated information.\n\n    {:node_update, {nodeName, status}}\n\nWhere status can be `:offline` or `:online`, and nodeName is the URI of the node.\nCoreServer manages the monitoring and de-monitoring nodes in Beethoven as they go through their lifecycle.\n\n---\n\n## `RoleMgmt`\n\nA framework for distributing work across a Beethoven cluster.\nRoles are defined in the `:beethoven` application configuration.\nThese roles are hosted on the nodes that come into the cluster, and redistributed when they fail.\n\nFor more information on using RoleMgmt, please see the module's documentation.\n\n---\n\n## `Allocator`\n\nA signal based work allocator that helps determine how busy a given Beethoven cluster node is.\nUsing these signals, it determines how busy a node is, and assigns a float.\nThe smaller the float, the less busy a node is considered.\n\nUsing `Beethoven.Allocator.Agent`, you gain access to the `signal()` macro.\n\n    signal(name: :http_connections, weight: 10.0, type: :count)\n\nThis allows you to create your own signals to feed into the allocator.\nThis method allows the allocator to be flexible, and signals to be declared and used as needed.\n\nCalling `Beethoven.Allocator.allocate/0` will check the Mnesia table used by the Allocator, and return the lest-busy node.\n\n---\n\n## `BeaconServer`\n\nThis PID runs a TCP-Socket server to help new Beethoven nodes find each other.\nThe port for listener can be set in the application configuration for `Beethoven`.\nUsing this port, the `Locator` service within the other nodes can attempt to find each other.\n\nThis module utilizes `Beethoven.SeekChat` for (de)serialization of TCP calls.\n\n---\n\n## `Locator`\n\nThis PID will enumerate all IPs within a given subnet; checking if they are running an instance of `BeaconServer`.\nThis PID will enumerate the IPs 3 times.\nIf nothing is found after the 3rd attempt, the cluster start as-is in :standalone mode.\n\nThis module utilizes `Beethoven.SeekChat` for (de)serialization of TCP calls.\n\n---\n\n## `HwMon`\n\nSet of PIDs to monitor and record historical hardware consumption.\nWill sample CPU and RAM usage every 5 seconds and save up to 100k samples per metric.\nWhen resources are sampled, a signal to `Allocator` is called and the utilization is logged.","ref":"Beethoven.html"},{"type":"function","title":"Beethoven.allocate/0","doc":"Returns the least-busy node from the Allocator's Mnesia table.\n\nWrapper for `Beethoven.Allocator.allocate/0`.","ref":"Beethoven.html#allocate/0"},{"type":"function","title":"Beethoven.get_active_cluster_nodes/0","doc":"Returns all active cluster nodes by their Node URI.","ref":"Beethoven.html#get_active_cluster_nodes/0"},{"type":"function","title":"Beethoven.get_cluster_nodes/0","doc":"Returns the Node URIs of all nodes that have joined the Beethoven Cluster","ref":"Beethoven.html#get_cluster_nodes/0"},{"type":"function","title":"Beethoven.get_detailed_cluster_nodes/0","doc":"Returns all cluster nodes and their state.\n\n# Example\n\n    [%{node: node(), status: :online | :offline, last_change: DateTime.t()}]","ref":"Beethoven.html#get_detailed_cluster_nodes/0"},{"type":"function","title":"Beethoven.get_node_status/0","doc":"Returns state of this node within the Beethoven cluster.\nReturn will only be `:clustered` or `:standalone`.\n\nWrapper for `Beethoven.CoreServer.get_mode/0`.","ref":"Beethoven.html#get_node_status/0"},{"type":"function","title":"Beethoven.get_roles/0","doc":"Returns all Beethoven hosted roles with their associated metadata.","ref":"Beethoven.html#get_roles/0"},{"type":"type","title":"Beethoven.nodeList/0","doc":"List of node names.","ref":"Beethoven.html#t:nodeList/0"},{"type":"type","title":"Beethoven.nodeStatusMap/0","doc":"Status of a Beethoven cluster node.","ref":"Beethoven.html#t:nodeStatusMap/0"},{"type":"type","title":"Beethoven.nodeStatusMapList/0","doc":"List of `nodeStatus()` objects.","ref":"Beethoven.html#t:nodeStatusMapList/0"},{"type":"function","title":"Beethoven.ready?/0","doc":"Gets the current ready state for Beethoven.\n\nWrapper for `Beethoven.Ready.ready?/0`.","ref":"Beethoven.html#ready?/0"},{"type":"type","title":"Beethoven.role_definition/0","doc":"Role data structure.","ref":"Beethoven.html#t:role_definition/0"},{"type":"type","title":"Beethoven.roleDefinitionList/0","doc":"List of `role()`.","ref":"Beethoven.html#t:roleDefinitionList/0"},{"type":"function","title":"Beethoven.until_ready/1","doc":"Similar to `ready?()` but will block until the service is ready.\nDefaults to 5_000 milliseconds.\n\nWrapper for `Beethoven.Ready.until_ready/1`.","ref":"Beethoven.html#until_ready/1"},{"type":"function","title":"Beethoven.whois_hosting/1","doc":"Returns what nodes host a given role.","ref":"Beethoven.html#whois_hosting/1"},{"type":"module","title":"Beethoven.Allocator","doc":"Allocator is a stack of processes that facilitate the aggregation of telemetry signals\nto determine how busy a given `Beethoven` node is.\n\n# Public API\n\n- `allocate/0` Provides the name of the least-busy node in the cluster.\n- `allocation_list/0` Provides a sorted list of all the nodes in the mnesia cluster. Nodes sorted from least-to-most busy.\n- `get_all/0` Dumps a sorted list of all records from the `AllocTracker` mnesia table. Records sorted from least-to-most busy.\n\n# Readme\n\nThe busyness of nodes is determined via signals.\nA signal is defined via the `signal/1` macro in `Elixir.Beethoven.Allocator.Agent`","ref":"Beethoven.Allocator.html"},{"type":"module","title":"Example - Beethoven.Allocator","doc":"defmodule HttpSignal do\n      use Elixir.Beethoven.Allocator.Agent\n      signal(name: :http_connections, weight: 10.0, type: :count)\n    end\n\n\nThis creates a function on compile time that is used to send the signal to the Allocator.","ref":"Beethoven.Allocator.html#module-example"},{"type":"module","title":"Example for `:count` type signals - Beethoven.Allocator","doc":"# Increases the internal count by 1\n      HttpSignal.increment_http_connections_count/0\n\n      # Decreases the internal count by 1\n      HttpSignal.decrement_http_connections_count/0\n\n# Signal types\n\n- `:count` -> Controls a counter for a given metric.\nCreates 2 functions. `increment_{:name}_count/0`\n- `:percent` -> Represents a percent value.\nCreates 1 function. `percent_{:name}/1`\n- `:pre_processed` -> Represents an abstract value.\nCreates 1 function. `pre_processed_{:name}/1`\n\n# Signal handling","ref":"Beethoven.Allocator.html#module-example-for-count-type-signals"},{"type":"module","title":"Ingress - Beethoven.Allocator","doc":"Once the signals are sent via the generated function, they are casted to a local instance of `Elixir.Beethoven.Allocator.Ingress`.\nThis service will normalize the data from the function and save to ETS. Once saved, it will signal `Elixir.Beethoven.Allocator.Cruncher` to check the new data.","ref":"Beethoven.Allocator.html#module-ingress"},{"type":"module","title":"Cruncher - Beethoven.Allocator","doc":"Once signaled, this service will call the ETS table shared with `Elixir.Beethoven.Allocator.Ingress` and grab all the current signal data.\nUsing the weight and data payload for the signal, a busy score is generated. This score is stored in an Mnesia table for all other nodes to access.\nThis flow allows other PIDs to call the public API for `Elixir.Beethoven.Allocator` and get the nodeURI for the node with the least amount of work on it.","ref":"Beethoven.Allocator.html#module-cruncher"},{"type":"function","title":"Beethoven.Allocator.alert_me/0","doc":"# CoreServer\n\nTell the local CoreServer that we want to be alerted to changes to cluster node state.\nUpdate will be sent in the form of a cast.","ref":"Beethoven.Allocator.html#alert_me/0"},{"type":"function","title":"Example - Beethoven.Allocator.alert_me/0","doc":"{:node_update, {nodeName, status}}","ref":"Beethoven.Allocator.html#alert_me/0-example"},{"type":"function","title":"Beethoven.Allocator.allocate/0","doc":"Provides the URI of the least-busy node in the cluster.\nUsing this function has no side effects so discarding the output without using it will not cause issues.","ref":"Beethoven.Allocator.html#allocate/0"},{"type":"function","title":"Beethoven.Allocator.allocation_list/0","doc":"Gets all the records from the Allocator table, but returns only the URIs of the nodes..\nRecords come pre-sorted from least-to-most busy.","ref":"Beethoven.Allocator.html#allocation_list/0"},{"type":"function","title":"Beethoven.Allocator.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Beethoven.Allocator.html#child_spec/1"},{"type":"function","title":"Beethoven.Allocator.dirty_select/1","doc":"Similar to `:mnesia.dirty_select/2` but only needs the match spec as an argument.\nThe table name of the DistrServer `Elixir.Beethoven.Allocator`'s mnesia table is input automatically as the 1st arity.","ref":"Beethoven.Allocator.html#dirty_select/1"},{"type":"function","title":"Beethoven.Allocator.fetch/1","doc":"Fetches data from the DistrServer `Elixir.Beethoven.Allocator`'s mnesia table.\nUses a record key to query the data. Will return all matching records.","ref":"Beethoven.Allocator.html#fetch/1"},{"type":"function","title":"Beethoven.Allocator.fetch_all/0","doc":"Fetches all records from the DistrServer `Elixir.Beethoven.Allocator`'s mnesia table.","ref":"Beethoven.Allocator.html#fetch_all/0"},{"type":"function","title":"Beethoven.Allocator.get_all/0","doc":"Gets all the records from the Allocator table.\nRecords come pre-sorted from least-to-most busy.","ref":"Beethoven.Allocator.html#get_all/0"},{"type":"function","title":"Beethoven.Allocator.get_table_name/0","doc":"Returns the name of the DistrServer `Elixir.Beethoven.Allocator`'s mnesia table.","ref":"Beethoven.Allocator.html#get_table_name/0"},{"type":"function","title":"Beethoven.Allocator.start_link/1","doc":"Supervisor Entry point.","ref":"Beethoven.Allocator.html#start_link/1"},{"type":"function","title":"Beethoven.Allocator.subscribe/1","doc":"Subscribes to the table mapped to the DistrServer `Elixir.Beethoven.Allocator`'s mnesia table.\n\n# Matches based on subscription\n\n## `:simple`\n\n    {:mnesia_table_event, {:atom, record(), _op_data}}\n\n## `:detailed`\n\n    {:mnesia_table_event, {:atom, module() | :atom(), record(), [] | [record()], _op_data}}","ref":"Beethoven.Allocator.html#subscribe/1"},{"type":"function","title":"Beethoven.Allocator.table_exists?/0","doc":"Checks if the DistrServer `Elixir.Beethoven.Allocator`'s mnesia table exists.","ref":"Beethoven.Allocator.html#table_exists?/0"},{"type":"function","title":"Beethoven.Allocator.until_exists/3","doc":"Holds the thread until the DistrServer `Elixir.Beethoven.Allocator`'s mnesia table becomes available, or timeout occurs.\nDefaults to `1_000` milliseconds for timeouts and `15` milliseconds for checking intervals.","ref":"Beethoven.Allocator.html#until_exists/3"},{"type":"module","title":"Beethoven.Allocator.Agent","doc":"Behaviour to define signals via the `signal()` macro.\n\nThis macro defines a signal that can be sent to the `Beethoven.Allocator` stack.\nOnce defined, this module will have compile-time created functions to streamline the use of the signal.\nEach signal has an atomic name, priority weight, and type defined in it.\nThis data is aggregated and used to generate a busy-score for a node.\n\n# Types\n- `:count` Counts the number of a given thing. Generates 2 functions.\nOne to increment the counter, the other to decrement it. Both modify the counter by `+/-1.0`.\nThe counter will never be lower then `0`.\n\n- `:percent` Represents a float between 0.0 and >100.0. This generates 1 function.\nThis function allows you to set a float payload in the signal.\nThis value must be a positive float.\n\n# Examples\n    defmodule Signals do\n      use Beethoven.Allocator.Agent\n\n      signal(name: :ram, weight: 10.0, type: :percent)\n      signal(name: :http_requests, weight: 5.0, type: :count)\n    end\n\n    # Generates\n    Signals.percent_ram(data) # Data is now the value of the metric in `:ets`\n    Signals.increment_http_requests_count() # +1.0\n    Signals.decrement_http_requests_count() # -1.0\n\nSee `Beethoven.Allocator.Cruncher` for information on how the signals are aggregated.","ref":"Beethoven.Allocator.Agent.html"},{"type":"function","title":"Beethoven.Allocator.Agent.send_signal/1","doc":"**DO NOT USE THIS DIRECTLY.  Please utilize the `signal()` macro within the `Beethoven.Allocator.Agent` module.**\n\nSends a signal message to `Allocator.Ingress`.\n\n# Payload schema\n\n    {header :: {name :: atom(), weight :: integer(), type :: atom()}, payload :: signal_payload()}","ref":"Beethoven.Allocator.Agent.html#send_signal/1"},{"type":"macro","title":"Beethoven.Allocator.Agent.signal/1","doc":"This macro defines a signal that can be sent to the `Beethoven.Allocator` stack.\nOnce defined, this module will have compile-time created functions to streamline the use of the signal.\nEach signal has an atomic name, priority weight, and type defined in it.\nThis data is aggregated and used to generate a busy-score for a node.\n\n# Types\n- `:count` Counts the number of a given thing. Generates 2 functions.\nOne to increment the counter, the other to decrement it. Both modify the counter by `+/-1.0`.\nThe counter will never be lower then `0`.\n\n- `:percent` Represents a float between 0.0 and >100.0. This generates 1 function.\nThis function allows you to set a float payload in the signal.\nThis value must be a positive float.\n\n# Examples\n    defmodule Signals do\n      use Beethoven.Allocator.Agent\n\n      signal(name: :ram, weight: 10.0, type: :percent)\n      signal(name: :http_requests, weight: 5.0, type: :count)\n    end\n\n    # Generates\n    Signals.percent_ram(data) # Data is now the value of the metric in `:ets`\n    Signals.increment_http_requests_count() # +1.0\n    Signals.decrement_http_requests_count() # -1.0\n\nSee `Beethoven.Allocator.Cruncher` for information on how the signals are aggregated.","ref":"Beethoven.Allocator.Agent.html#signal/1"},{"type":"type","title":"Beethoven.Allocator.Agent.signal_definition/0","doc":"Defines a single signal. Commonly used as an input for the `signal()` macro.","ref":"Beethoven.Allocator.Agent.html#t:signal_definition/0"},{"type":"type","title":"Beethoven.Allocator.Agent.signal_message/0","doc":"Message wrapper for sending payload to `Beethoven.Allocator.Ingress`.","ref":"Beethoven.Allocator.Agent.html#t:signal_message/0"},{"type":"type","title":"Beethoven.Allocator.Agent.signal_message_header/0","doc":"Header for the signal metadata; when sending a signal to `Beethoven.Allocator.Ingress`.","ref":"Beethoven.Allocator.Agent.html#t:signal_message_header/0"},{"type":"type","title":"Beethoven.Allocator.Agent.signal_type/0","doc":"Defines types of signal that can be used.","ref":"Beethoven.Allocator.Agent.html#t:signal_type/0"},{"type":"module","title":"Beethoven.Allocator.Cruncher","doc":"Crunches signal data and generates an allocation (or busy-score) score for the current node.\nSignal data is pulled from the `Beethoven.Allocator.Ingress.Cache` ets table.\nThe aggregated score is stored in the `Beethoven.Allocator.Tracker` Mnesia table.\nThis PID will refresh the score anytime `Beethoven.Allocator.Ingress` pushes an update and sends a cast to this PID.","ref":"Beethoven.Allocator.Cruncher.html"},{"type":"function","title":"Beethoven.Allocator.Cruncher.algorithm/1","doc":"Crunching algorithm for each signal item.","ref":"Beethoven.Allocator.Cruncher.html#algorithm/1"},{"type":"function","title":"Beethoven.Allocator.Cruncher.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Beethoven.Allocator.Cruncher.html#child_spec/1"},{"type":"function","title":"Beethoven.Allocator.Cruncher.send_check/0","doc":"Sends a check message to the local instance of Cruncher","ref":"Beethoven.Allocator.Cruncher.html#send_check/0"},{"type":"function","title":"Beethoven.Allocator.Cruncher.start_link/1","doc":"Supervisor Entry point.","ref":"Beethoven.Allocator.Cruncher.html#start_link/1"},{"type":"module","title":"Beethoven.Allocator.Ingress","doc":"GenServer to handle ingress of signal data.\nData sent to this Server is pushed into `:ets` for use by `Beethoven.Allocator.Cruncher`.","ref":"Beethoven.Allocator.Ingress.html"},{"type":"function","title":"Beethoven.Allocator.Ingress.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Beethoven.Allocator.Ingress.html#child_spec/1"},{"type":"type","title":"Beethoven.Allocator.Ingress.signal_message/0","doc":"Message wrapper for sending payload to `Beethoven.Allocator.Ingress`.","ref":"Beethoven.Allocator.Ingress.html#t:signal_message/0"},{"type":"type","title":"Beethoven.Allocator.Ingress.signal_message_header/0","doc":"Header for the signal metadata; when sending a signal to `Beethoven.Allocator.Ingress`.","ref":"Beethoven.Allocator.Ingress.html#t:signal_message_header/0"},{"type":"function","title":"Beethoven.Allocator.Ingress.start_link/1","doc":"Supervisor Entry point.","ref":"Beethoven.Allocator.Ingress.html#start_link/1"},{"type":"module","title":"Beethoven.Az","doc":"# WORK IN PROGRESS!\n\nAzure Platform awareness for the node.\nIf the app is not in Azure, genserver will response `:no_azure` to all calls.\n\n# Functions\n- `get_state/0` Returns the entire state of the GenServer.\n- `get_az_subnet/0` Returns the subnet this VM resides in. Pulls data from cache.\n- `get_az_region/0` Returns the region this VM resides in. Pulls data from cache.","ref":"Beethoven.Az.html"},{"type":"function","title":"Beethoven.Az.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Beethoven.Az.html#child_spec/1"},{"type":"function","title":"Beethoven.Az.get_az_region/0","doc":"Retrieves the Azure region from IMDS data.","ref":"Beethoven.Az.html#get_az_region/0"},{"type":"function","title":"Beethoven.Az.get_az_subnet/0","doc":"Retrieves the VM's networking config from IMDS data.\n\n**RETURNS ONLY PRIMARY INTERFACE FOR VM**","ref":"Beethoven.Az.html#get_az_subnet/0"},{"type":"function","title":"Beethoven.Az.get_state/0","doc":"Retrieves the Azure region from IMDS data.","ref":"Beethoven.Az.html#get_state/0"},{"type":"function","title":"Beethoven.Az.start_link/1","doc":"Entry point for Supervisors. Links calling PID this this child pid.","ref":"Beethoven.Az.html#start_link/1"},{"type":"module","title":"Beethoven.BeaconServer","doc":"Beacon Server is used to help other Beethoven node find the cluster.\nThis is the TCP Listener the `Beethoven.Locator` server will check for.\n\n---\n\n# External API calls\nThese are calls that can be made from external servers\n- `attempt_recover/0` -> casts a recover message to the Beacon server.\nWill restart the socket if in a failed state. Will do nothing otherwise.\n\n---\n\n# Notes\nThis TCP listener will communicate via the `Beethoven.SeekChat` module.\nThis module provides tools and types for interacting over raw TCP packets between the Beacon server and Locator.","ref":"Beethoven.BeaconServer.html"},{"type":"function","title":"Beethoven.BeaconServer.attempt_recover/0","doc":"Attempts a recovery operation on the Beacon Server.\nIf Server is in a failed state, it will attempt to reboot it.\nIf not, nothing will happen.\n\nIndication of the outcome will be found in the server logs for BeaconServer.","ref":"Beethoven.BeaconServer.html#attempt_recover/0"},{"type":"function","title":"Beethoven.BeaconServer.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Beethoven.BeaconServer.html#child_spec/1"},{"type":"function","title":"Beethoven.BeaconServer.start_link/1","doc":"Entry point for a supervisor.","ref":"Beethoven.BeaconServer.html#start_link/1"},{"type":"module","title":"Beethoven.CoreServer","doc":"Core Service for Beethoven.\nWatches other nodes within the cluster and cascade updates across the beethoven PIDs as needed.\n\n---\n\n# External API calls\nThese are calls that can be made from external servers\n- `alert_me/1` -> Alerts the caller on cluster node changes. (see 'Listening for cluster node changes' for more info.)\n- `new_node/1` -> Adds node to CoreServer state mnesia table. (Should be called by the `Listener` server)\n- `get_mode/0` -> Returns the mode of the CoreServer. (:standalone | :clustered)\n\n---\n\n# Listening for cluster node changes\n\nIf you invoke this module via `use`, it will import a specialized version of `alert_me/1` called `alert_me/0`.\nThe only major difference is that the name of the module is input into the first parameter of `alert_me/1`.","ref":"Beethoven.CoreServer.html"},{"type":"module","title":"Example - Beethoven.CoreServer","doc":"defmodule TestMod do\n      use Elixir.Beethoven.CoreServer\n\n      def fun() do\n        :ok = alert_me()\n      end\n\n    end\n\n\nUsing `alert_me/1` from a local client, you can tell the CoreServer to call you back when there is a change to a cluster node.\nIgnores changes to itself, only delivers updates of other nodes.\n\nTo use this, the caller *must* implement the `CoreServer` behavior and callback `node_update/2`.\nOnce a change occurs, the CoreServer will call the callback function for the following module/process.\n`node_update/2` should contain the logic needed when a node changes state.","ref":"Beethoven.CoreServer.html#module-example"},{"type":"macro","title":"Beethoven.CoreServer.__using__/1","doc":"Imports the `alert_me/0` function into the module with special customizations for the declaring module.","ref":"Beethoven.CoreServer.html#__using__/1"},{"type":"function","title":"Beethoven.CoreServer.add_node_to_mnesia/1","doc":"Adds a node to Mnesia cluster","ref":"Beethoven.CoreServer.html#add_node_to_mnesia/1"},{"type":"function","title":"Beethoven.CoreServer.alert_me/1","doc":"Tell the local CoreServer that we want to be alerted to changes to cluster node state.\nUpdate will be sent in the form of a cast.","ref":"Beethoven.CoreServer.html#alert_me/1"},{"type":"function","title":"Example - Beethoven.CoreServer.alert_me/1","doc":"{:node_update, {nodeName, status}}","ref":"Beethoven.CoreServer.html#alert_me/1-example"},{"type":"function","title":"Beethoven.CoreServer.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Beethoven.CoreServer.html#child_spec/1"},{"type":"function","title":"Beethoven.CoreServer.dirty_select/1","doc":"Similar to `:mnesia.dirty_select/2` but only needs the match spec as an argument.\nThe table name of the DistrServer `Elixir.Beethoven.CoreServer`'s mnesia table is input automatically as the 1st arity.","ref":"Beethoven.CoreServer.html#dirty_select/1"},{"type":"function","title":"Beethoven.CoreServer.entry_point/1","doc":"Callback for entry when we are in :standalone or :clustered mode","ref":"Beethoven.CoreServer.html#entry_point/1"},{"type":"function","title":"Beethoven.CoreServer.fetch/1","doc":"Fetches data from the DistrServer `Elixir.Beethoven.CoreServer`'s mnesia table.\nUses a record key to query the data. Will return all matching records.","ref":"Beethoven.CoreServer.html#fetch/1"},{"type":"function","title":"Beethoven.CoreServer.fetch_all/0","doc":"Fetches all records from the DistrServer `Elixir.Beethoven.CoreServer`'s mnesia table.","ref":"Beethoven.CoreServer.html#fetch_all/0"},{"type":"function","title":"Beethoven.CoreServer.get_mode/0","doc":"Gets mode from the CoreServer.","ref":"Beethoven.CoreServer.html#get_mode/0"},{"type":"function","title":"Beethoven.CoreServer.get_table_name/0","doc":"Returns the name of the DistrServer `Elixir.Beethoven.CoreServer`'s mnesia table.","ref":"Beethoven.CoreServer.html#get_table_name/0"},{"type":"function","title":"Beethoven.CoreServer.handle_cast/2","doc":"Callback to handle casts for services that need updates on node state.","ref":"Beethoven.CoreServer.html#handle_cast/2"},{"type":"function","title":"Beethoven.CoreServer.new_node/1","doc":"Add a node to the Cluster Node tracker.\nIf node is already tracked, it will be marked as `:online`.","ref":"Beethoven.CoreServer.html#new_node/1"},{"type":"type","title":"Beethoven.CoreServer.nodeStatus/0","doc":"Possible status(s) for nodes within Beethoven.\n# Options\n- `:online`\n- `:offline`","ref":"Beethoven.CoreServer.html#t:nodeStatus/0"},{"type":"type","title":"Beethoven.CoreServer.serverStatus/0","doc":"Possible statuses for CoreServer","ref":"Beethoven.CoreServer.html#t:serverStatus/0"},{"type":"function","title":"Beethoven.CoreServer.start_link/1","doc":"Supervisor Entry point.","ref":"Beethoven.CoreServer.html#start_link/1"},{"type":"function","title":"Beethoven.CoreServer.subscribe/1","doc":"Subscribes to the table mapped to the DistrServer `Elixir.Beethoven.CoreServer`'s mnesia table.\n\n# Matches based on subscription\n\n## `:simple`\n\n    {:mnesia_table_event, {:atom, record(), _op_data}}\n\n## `:detailed`\n\n    {:mnesia_table_event, {:atom, module() | :atom(), record(), [] | [record()], _op_data}}","ref":"Beethoven.CoreServer.html#subscribe/1"},{"type":"function","title":"Beethoven.CoreServer.table_exists?/0","doc":"Checks if the DistrServer `Elixir.Beethoven.CoreServer`'s mnesia table exists.","ref":"Beethoven.CoreServer.html#table_exists?/0"},{"type":"type","title":"Beethoven.CoreServer.trackerEvent/0","doc":"Single tracker event from the Mnesia table","ref":"Beethoven.CoreServer.html#t:trackerEvent/0"},{"type":"type","title":"Beethoven.CoreServer.trackerRow/0","doc":"A single row in the CoreServer tracker.","ref":"Beethoven.CoreServer.html#t:trackerRow/0"},{"type":"function","title":"Beethoven.CoreServer.until_exists/3","doc":"Holds the thread until the DistrServer `Elixir.Beethoven.CoreServer`'s mnesia table becomes available, or timeout occurs.\nDefaults to `1_000` milliseconds for timeouts and `15` milliseconds for checking intervals.","ref":"Beethoven.CoreServer.html#until_exists/3"},{"type":"behaviour","title":"Beethoven.DistrServer","doc":"Distr(ibuted)Server is a modified `GenServer` that allows for seamless integration with a dedicated Mnesia table.\nThis was specially built for operation in a Beethoven environment.\nThe idea is that the brain of the genserver can be set with mnesia and not the GenServer's internal state.\nThis allows for the compute and state of the genserver to be distributed across the Beethoven cluster.\n\nSupports all GenServer logic and callbacks except for `init/1`.\nUse of this callback directly will cause unexpected errors and behaviour.\nThe entry point for the DistrServer is `entry_point/1`.\n`entry_point/1` is identical to `init/1` in terms of both input and return types.\n\n# Example\n\n    defmodule Test do\n\n      use DistrServer, subscribe?: true\n\n      # Standard OTP entry point for starting the PID.\n      def start_link(init_args) do\n        DistrServer.start_link(__MODULE__, init_args, name: __MODULE__)\n      end\n\n      # The configuration for the DistrServer's Mnesia table.\n      @impl true\n      def config() do\n        %{\n          tableName: TestTracker,\n          columns: [:col1, :col2, :last_change],\n          indexes: [],\n          dataType: :set,\n          copyType: :multi\n        }\n      end\n\n      # This is ran when the table is newly created.\n      @impl true\n      def create_action(_tableConfig) do\n        ...\n        :ok\n      end\n\n      # Similar to GenServer.init/1.\n      @impl true\n      def entry_point(_init_args) do\n        ...\n        {:ok, :ok} # all init/1 returns are supported.\n      end\n\n    end\n\n---\n\n# Breakdown\n\n- `use DistrServer, subscribe?: boolean()` -> This implements the DistrServer behaviour and mnesia tool functions.\n    - `:subscribe?` Utilizing this required parameter will tell the compiler if the DistrServer should automatically subscribe to the Mnesia table it is entangled with.\n        Subscribing to the table **will** copy it to the local memory of the node running this DistrServer.\n\n\n- `DistrServer.start_link/1` -> Similar to `GenServer.start_link/1`. Supports all the same arguments and arty.\n\n- **Callback:** `config/0` -> Provides the configuration that will be used by the DistrServer's Mnesia table.\n    See the section below on `config/0` for more information\n\n- **Callback:** `create_action/1` -> Logic ran if the DistrServer has to create the Mnesia table.\n    This callback is optional, but with it, you can pre-fill the mnesia table with data as needed.\n    If the DistrServer boots and the table already exists in the cluster, this function will not be called.\n\n- **Callback:** `entry_point/1` -> Similar to `GenServer.init/1`. Supports all the same arguments and arty.\n    With DistrServer, the `init/1` callback for GenServers is used in the creation of the Mnesia table.\n\n---\n\n# `config/0`\n\n`config/0` is a callback that requires the returns of a specific map.\n\n# Example\n\n      def config() do\n      %{\n        tableName: module() | atom(),\n        columns: list(atom()),\n        indexes: list(atom()),\n        dataType: :set | :ordered_set | :bag | :duplicate_bag,\n        copyType: :multi | :local\n      }\n    end\n\n- `:tableName` The name that will be used when creating the Mnesia table.\n\n- `:columns` The columns that will be used to create the Mnesia table.\n    The first item in the list will be considered the `key` for the record\n\n- `:indexes` Defines additional columns that need to be indexed.\n    By default, the key for the record is indexed.\n    Adding the key column to the list will result in an error for Mnesia.\n\n- `:dataType` This is the type of Mnesia table we are creating.\n    The types allowed are the same ones supported by Mnesia.\n\n- `:copyType` This defines if the DistrServer should copy the table to the local memory of the node.\n    By default, if the DistrServer creates the Mnesia table, it will always be saved to memory.\n    This option comes into effect when the DistrServer is joining an existing cluster where this role is already hosted.\n    If your DistrServer is set to subscribe to the mnesia table, it will be copied to memory as is required by Mnesia.\n    If you are not subscribing, your options are `:local` and `:multi`.\n\n    - `:local` will only keep the table in the memory of the original node.\n    This mode is dangerous as losing the original node will result in both data loss, and potential corruption.\n    In the event of a failover in this mode, the failing over DistrServer will **not** run `create_action/1` as the table will be marked as created.\n\n    - `:multi` will copy the table to the local memory of any node running the DistrServer.\n    This mode is recommended for most use cases.\n    Please check into how Mnesia operations (transaction or dirty) work so you use this most efficiently.\n    Failure to do so may lead to poor write performance.","ref":"Beethoven.DistrServer.html"},{"type":"function","title":"Beethoven.DistrServer.call/3","doc":"Sends a cast to the provided `DistrServer`.\nSimilar to `GenServer.call/2` and `GenServer.call/3`","ref":"Beethoven.DistrServer.html#call/3"},{"type":"function","title":"Beethoven.DistrServer.cast/2","doc":"Sends a cast to the provided `DistrServer`. Similar to `GenServer.cast/2`","ref":"Beethoven.DistrServer.html#cast/2"},{"type":"callback","title":"Beethoven.DistrServer.config/0","doc":"**-Callback required-**\n\n`DistrServer` configuration. See `distrConfig()` type for more information on the return.","ref":"Beethoven.DistrServer.html#c:config/0"},{"type":"type","title":"Beethoven.DistrServer.copyTypes/0","doc":"Copy options for the Mnesia table.\n- `:local` -> Copies are only on the table-creating-node.\n- `:multi` -> Copies are pushed to ALL nodes in the cluster.","ref":"Beethoven.DistrServer.html#t:copyTypes/0"},{"type":"callback","title":"Beethoven.DistrServer.create_action/1","doc":"**-Callback required-**\n\nCallback that is triggered when the process creates the Mnesia Table for the cluster.","ref":"Beethoven.DistrServer.html#c:create_action/1"},{"type":"function","title":"Beethoven.DistrServer.distr_to_table_conf/1","doc":"Converts the `DistrConfig()` into the `tableConfig()` type.","ref":"Beethoven.DistrServer.html#distr_to_table_conf/1"},{"type":"type","title":"Beethoven.DistrServer.distrConfig/0","doc":"Configuration for the `DistrServer` instance.\n- `:tableName` -> Atomic name for the table.\n- `:columns` -> List of atoms representing the names of columns in the name.\n- `:indexes` -> List of table columns that should be indexed.\n**Note:** Indexing a column will slow writes to it,\nbut make read operations consistent regardless of the table's size.\n- `:dataType` -> Data type for the Mnesia table.\n- `:copyType` -> How the new table will be copied across the Beethoven cluster.","ref":"Beethoven.DistrServer.html#t:distrConfig/0"},{"type":"callback","title":"Beethoven.DistrServer.entry_point/1","doc":"**-Callback required-**\n\nEntry point for the `DistrServer` process. Similar to `init/1` for GenServers.","ref":"Beethoven.DistrServer.html#c:entry_point/1"},{"type":"function","title":"Beethoven.DistrServer.start_link/3","doc":"Starts a `DistrServer` process under the supervisor tree.\nSimilar to `GenServer.start_link/2` and `GenServer.start_link/3`","ref":"Beethoven.DistrServer.html#start_link/3"},{"type":"module","title":"Beethoven.HwMon","doc":"Stack of services to help check system utilization and push sampled data into the `Beethoven.Allocator` stack of services.\n\n# Public API\n\n- `get_hw_usage/0` Dumps the entire hardware usage Mnesia table.\n\n# Readme\n\nThis module contains 2 primary services:\n\n- `Server` This service will take samples of system performance and send the data to multiple services within Beethoven.\nThe most current data is stored in an Mnesia table.\n- `Archive` Archive hosts historical performance signals.\nWill store up to 100k entries *per metric*. This is a hard limit. Older logs will be pruned in a FIFO manor.\nThis data is stored in an ETS table on the local node.","ref":"Beethoven.HwMon.html"},{"type":"function","title":"Beethoven.HwMon.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Beethoven.HwMon.html#child_spec/1"},{"type":"function","title":"Beethoven.HwMon.get_hw_usage/0","doc":"Get Node performance specs from Mnesia.\n\n    {Beethoven.HwMon.Server.Tracker, :node, :avail_cpu, :avail_ram_gb, :last_change}","ref":"Beethoven.HwMon.html#get_hw_usage/0"},{"type":"function","title":"Beethoven.HwMon.start_link/1","doc":"Supervisor Entry point.","ref":"Beethoven.HwMon.html#start_link/1"},{"type":"module","title":"Beethoven.HwMon.Archive","doc":"Tracks historical CPU and RAM usage as sampled by `HwMon.Server`.\nEach metric will only cache up to 100k entries. Entries are pruned via FIFO.","ref":"Beethoven.HwMon.Archive.html"},{"type":"function","title":"Beethoven.HwMon.Archive.cast_cpu/1","doc":"Casts CPU telemetry to Archive.","ref":"Beethoven.HwMon.Archive.html#cast_cpu/1"},{"type":"function","title":"Beethoven.HwMon.Archive.cast_diskIO/1","doc":"Casts DiskIO telemetry to Archive.","ref":"Beethoven.HwMon.Archive.html#cast_diskIO/1"},{"type":"function","title":"Beethoven.HwMon.Archive.cast_networkIO/1","doc":"Casts networkIO telemetry to Archive.","ref":"Beethoven.HwMon.Archive.html#cast_networkIO/1"},{"type":"function","title":"Beethoven.HwMon.Archive.cast_ram/1","doc":"Casts RAM telemetry to Archive.","ref":"Beethoven.HwMon.Archive.html#cast_ram/1"},{"type":"function","title":"Beethoven.HwMon.Archive.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Beethoven.HwMon.Archive.html#child_spec/1"},{"type":"function","title":"Beethoven.HwMon.Archive.get_stat/1","doc":"Gets telemetry for a given stat.","ref":"Beethoven.HwMon.Archive.html#get_stat/1"},{"type":"function","title":"Beethoven.HwMon.Archive.start_link/1","doc":"Supervisor Entry point.","ref":"Beethoven.HwMon.Archive.html#start_link/1"},{"type":"type","title":"Beethoven.HwMon.Archive.stat_category/0","doc":"Stat categories.","ref":"Beethoven.HwMon.Archive.html#t:stat_category/0"},{"type":"type","title":"Beethoven.HwMon.Archive.stat_object/0","doc":"Stat object that is stored on ETS","ref":"Beethoven.HwMon.Archive.html#t:stat_object/0"},{"type":"module","title":"Beethoven.HwMon.Server","doc":"DistrServer to monitor hardware resources and sends signals to `Beethoven.Allocator` plus the integrated Mnesia table.","ref":"Beethoven.HwMon.Server.html"},{"type":"function","title":"Beethoven.HwMon.Server.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Beethoven.HwMon.Server.html#child_spec/1"},{"type":"function","title":"Beethoven.HwMon.Server.dirty_select/1","doc":"Similar to `:mnesia.dirty_select/2` but only needs the match spec as an argument.\nThe table name of the DistrServer `Elixir.Beethoven.HwMon.Server`'s mnesia table is input automatically as the 1st arity.","ref":"Beethoven.HwMon.Server.html#dirty_select/1"},{"type":"function","title":"Beethoven.HwMon.Server.fetch/1","doc":"Fetches data from the DistrServer `Elixir.Beethoven.HwMon.Server`'s mnesia table.\nUses a record key to query the data. Will return all matching records.","ref":"Beethoven.HwMon.Server.html#fetch/1"},{"type":"function","title":"Beethoven.HwMon.Server.fetch_all/0","doc":"Fetches all records from the DistrServer `Elixir.Beethoven.HwMon.Server`'s mnesia table.","ref":"Beethoven.HwMon.Server.html#fetch_all/0"},{"type":"function","title":"Beethoven.HwMon.Server.get_table_name/0","doc":"Returns the name of the DistrServer `Elixir.Beethoven.HwMon.Server`'s mnesia table.","ref":"Beethoven.HwMon.Server.html#get_table_name/0"},{"type":"function","title":"Beethoven.HwMon.Server.handle_cast/2","doc":"#\n# PERCENTAGE FORMAT\n#\n# YES -> 4% => 4.0 | 61% => 61.0\n# NO  -> 4% => 0.04 | 61% => 0.61","ref":"Beethoven.HwMon.Server.html#handle_cast/2"},{"type":"function","title":"Beethoven.HwMon.Server.start_link/1","doc":"Supervisor Entry point.","ref":"Beethoven.HwMon.Server.html#start_link/1"},{"type":"function","title":"Beethoven.HwMon.Server.subscribe/1","doc":"Subscribes to the table mapped to the DistrServer `Elixir.Beethoven.HwMon.Server`'s mnesia table.\n\n# Matches based on subscription\n\n## `:simple`\n\n    {:mnesia_table_event, {:atom, record(), _op_data}}\n\n## `:detailed`\n\n    {:mnesia_table_event, {:atom, module() | :atom(), record(), [] | [record()], _op_data}}","ref":"Beethoven.HwMon.Server.html#subscribe/1"},{"type":"function","title":"Beethoven.HwMon.Server.table_exists?/0","doc":"Checks if the DistrServer `Elixir.Beethoven.HwMon.Server`'s mnesia table exists.","ref":"Beethoven.HwMon.Server.html#table_exists?/0"},{"type":"function","title":"Beethoven.HwMon.Server.until_exists/3","doc":"Holds the thread until the DistrServer `Elixir.Beethoven.HwMon.Server`'s mnesia table becomes available, or timeout occurs.\nDefaults to `1_000` milliseconds for timeouts and `15` milliseconds for checking intervals.","ref":"Beethoven.HwMon.Server.html#until_exists/3"},{"type":"module","title":"Beethoven.Ipv4","doc":"Module to handle scanning IPV4 addresses within a network.","ref":"Beethoven.Ipv4.html"},{"type":"function","title":"Beethoven.Ipv4.get_host_network_addresses/0","doc":"Retrieves the hosts IP address. Pulls Network and Netmask from the config.","ref":"Beethoven.Ipv4.html#get_host_network_addresses/0"},{"type":"function","title":"Beethoven.Ipv4.get_host_network_addresses/2","doc":"Same as get_host_network_addresses/0 but works off provided network and netmask.","ref":"Beethoven.Ipv4.html#get_host_network_addresses/2"},{"type":"function","title":"Beethoven.Ipv4.get_hosts/2","doc":"Generates a list of hosts with the network and netmask provided","ref":"Beethoven.Ipv4.html#get_hosts/2"},{"type":"function","title":"Beethoven.Ipv4.get_netmask_hosts/1","doc":"Gets the number of IPs within a netmask.","ref":"Beethoven.Ipv4.html#get_netmask_hosts/1"},{"type":"function","title":"Beethoven.Ipv4.increment_ip/1","doc":"Increments IP by one host.","ref":"Beethoven.Ipv4.html#increment_ip/1"},{"type":"module","title":"Beethoven.Locator","doc":"GenServer to handle searching for other Beethoven nodes or clusters.\nThis Locator server's goal is to find a BeaconServer on another node.\nOnce connected, they will communicate with each other via the `Beethoven.SeekChat` module.\n\n# Modes\n- `:seeking` -> Currently Searching for listening servers.\n- `:watching` ->  (TBD) Slower search to ensure there are no standalone clusters or nodes.\n\n---\n\n# `:seeking`\nService will start up and perform a seeking operation.\nThe locator will start the Core service(s) (via Substrate) once either is met:\n- (1). Seeking attempts have become exhausted. [CoreServer <- `:standalone`]\n- (2). A listener server is found. [CoreServer <- `:clustered`]\n\n---\n\n# `:watching` (WIP)\nUsed when the cluster is in standalone mode.\nThis mode will continue the scan for other clusters.\nThis is needed to ensure there are no other clusters existing at the same time.","ref":"Beethoven.Locator.html"},{"type":"function","title":"Beethoven.Locator.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Beethoven.Locator.html#child_spec/1"},{"type":"function","title":"Beethoven.Locator.start_link/1","doc":"Entry point for a supervisor.","ref":"Beethoven.Locator.html#start_link/1"},{"type":"module","title":"Beethoven.MnesiaTools","doc":"Generic Library to simplify certain Mnesia tasks.","ref":"Beethoven.MnesiaTools.html"},{"type":"function","title":"Beethoven.MnesiaTools.copy_table/1","doc":"Copies a replica of a desired table to local node memory.\n\nThe table may already exist in memory for the following reasons:\n- The table was created for all nodes, and this node was already in the cluster.\n- The table was initialized on this node.\n- A node with the same name was successfully joined the cluster in the past, and power-cycled.","ref":"Beethoven.MnesiaTools.html#copy_table/1"},{"type":"function","title":"Beethoven.MnesiaTools.create_table_ifnot_exist/1","doc":"Creates table if it does not already exist.\nApplies indexes on creation.\nWill","ref":"Beethoven.MnesiaTools.html#create_table_ifnot_exist/1"},{"type":"function","title":"Beethoven.MnesiaTools.delete_copy/1","doc":"Removes the replica of a desired table from local memory.","ref":"Beethoven.MnesiaTools.html#delete_copy/1"},{"type":"function","title":"Beethoven.MnesiaTools.table_exists?/1","doc":"Check if a Mnesia table exists in the cluster.","ref":"Beethoven.MnesiaTools.html#table_exists?/1"},{"type":"type","title":"Beethoven.MnesiaTools.tableConfig/0","doc":"TableConfig contains the payload needed to create the state table needed by the process.","ref":"Beethoven.MnesiaTools.html#t:tableConfig/0"},{"type":"module","title":"Beethoven.Ready","doc":"Simple PID to track when Beethoven has fully initialized.\nUsually just called from RoleServer when it has ran out of work.\n\nThis is important as creating Mnesia tables before joining Beethoven will cause cluster conflicts.\n\n# Public API\n- 'ready?/0' Queries the ETS table used by this module to see if Beethoven is ready. Returns boolean.\n- 'until_ready/1' Similar to `ready?/0` but will block until Beethoven is ready.\n- `set_ready/1` Sets the ready state of Beethoven on ETS. Normally called by `Beethoven.RoleServer`","ref":"Beethoven.Ready.html"},{"type":"function","title":"Beethoven.Ready.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Beethoven.Ready.html#child_spec/1"},{"type":"function","title":"Beethoven.Ready.ready?/0","doc":"Gets the current ready state for Beethoven.","ref":"Beethoven.Ready.html#ready?/0"},{"type":"function","title":"Beethoven.Ready.set_ready/1","doc":"Sets ready state for Beethoven. Setting this value to true will make `ready?()` return true; indicating the service has initialized.\nSetting `false` performs the opposite.","ref":"Beethoven.Ready.html#set_ready/1"},{"type":"function","title":"Beethoven.Ready.start_link/1","doc":"Supervisor Entry point.","ref":"Beethoven.Ready.html#start_link/1"},{"type":"function","title":"Beethoven.Ready.until_ready/1","doc":"Similar to `ready?()` but will block until the service is ready.\nDefaults to 5_000 milliseconds.","ref":"Beethoven.Ready.html#until_ready/1"},{"type":"module","title":"Beethoven.RoleMgmt","doc":"The RoleMgmt stack is a collective of servers responsible for the management and allocation of specialized OTP complaint PIDs.\nThese specialized PIDs are referred to as roles.\nRoles are defined in in the application config for `:beethoven`.\nTo understand compatibility for using a module as a Beethoven role, please see the below section on **Role Compatibility**.","ref":"Beethoven.RoleMgmt.html"},{"type":"module","title":"Example - Beethoven.RoleMgmt","doc":"config :beethoven,\n      ...\n      roles: [\n        # { ,  ,  ,  }\n        {:test, Beethoven.TestRole, [arg1: \"arg1\"], 1}\n      ]\n\nBased on the definition for each role, the nodes in the `Beethoven` cluster will ensure the required amount of that service is running across the cluster.\nThe number of service instances needed for the role is defined with the `InstanceCount` element in the tuple.\nThese services are spread across the cluster based on the busyness of the nodes. Busyness is determined by `Beethoven.Allocator`.\n\nIn the event of a node failure, the stack will react and re-distribute the roles to the remaining nodes in the cluster.\nIf a new/previous node comes online in the cluster, role assignment evaluation will be triggered to ensure the correct amount of roles are allocated.\nAs of now, roles are not balanced after they are assigned. Once the node is running a role, it runs it until it shuts down.\n\n# Role Compatibility\n\nMost OTP compliant PIDs are supported, but some caveats should be expected.\nThe major caveat is that calls across the Beethoven cluster are not supported using the `GenServer` or `DistrServer` cast or calls.\nTo communicate with these role PIDs across the Beethoven node level, you will need to use the erlang built-in `send/2` and `receive/1`.\nThese functions do support calling PIDs across the cluster.\n\nKeeping this in mind, there is a workaround.\nUsing a combination of the roles defined in RoleMgmt and `Beethoven.Utils.remote_call/3`,\nyou can make any GenServer or DistrServer function needed.\n\nThe `DistrServer` using modules were partially created for use with RoleMgmt.\nKeeping at least 2 instances of the DistrServer on the cluster helps ensure the data is resilient.\nHowever instances 3+ is recommended.\n\nIf you want to put the role on all nodes, please avoid utilizing RoleMgmt.\nInstead use a supervisor or application to host the PID instead.","ref":"Beethoven.RoleMgmt.html#module-example"},{"type":"function","title":"Beethoven.RoleMgmt.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Beethoven.RoleMgmt.html#child_spec/1"},{"type":"function","title":"Beethoven.RoleMgmt.start_link/1","doc":"Supervisor Entry point.","ref":"Beethoven.RoleMgmt.html#start_link/1"},{"type":"module","title":"Beethoven.RoleMgmt.Assign","doc":"Server to track nodes and their role assignments.\nOnce boot, the service will assign one of each open roles to itself.\nThese roles are cast to the `Manager` service to spawn the roles.\n\nFailover of roles across the cluster is handled by the `Failover` service.","ref":"Beethoven.RoleMgmt.Assign.html"},{"type":"function","title":"Beethoven.RoleMgmt.Assign.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Beethoven.RoleMgmt.Assign.html#child_spec/1"},{"type":"function","title":"Beethoven.RoleMgmt.Assign.dirty_select/1","doc":"Similar to `:mnesia.dirty_select/2` but only needs the match spec as an argument.\nThe table name of the DistrServer `Elixir.Beethoven.RoleMgmt.Assign`'s mnesia table is input automatically as the 1st arity.","ref":"Beethoven.RoleMgmt.Assign.html#dirty_select/1"},{"type":"function","title":"Beethoven.RoleMgmt.Assign.fetch/1","doc":"Fetches data from the DistrServer `Elixir.Beethoven.RoleMgmt.Assign`'s mnesia table.\nUses a record key to query the data. Will return all matching records.","ref":"Beethoven.RoleMgmt.Assign.html#fetch/1"},{"type":"function","title":"Beethoven.RoleMgmt.Assign.fetch_all/0","doc":"Fetches all records from the DistrServer `Elixir.Beethoven.RoleMgmt.Assign`'s mnesia table.","ref":"Beethoven.RoleMgmt.Assign.html#fetch_all/0"},{"type":"function","title":"Beethoven.RoleMgmt.Assign.get_table_name/0","doc":"Returns the name of the DistrServer `Elixir.Beethoven.RoleMgmt.Assign`'s mnesia table.","ref":"Beethoven.RoleMgmt.Assign.html#get_table_name/0"},{"type":"type","title":"Beethoven.RoleMgmt.Assign.record/0","doc":"Represents a single record in the Mnesia table.","ref":"Beethoven.RoleMgmt.Assign.html#t:record/0"},{"type":"function","title":"Beethoven.RoleMgmt.Assign.start_link/1","doc":"Supervisor Entry point.","ref":"Beethoven.RoleMgmt.Assign.html#start_link/1"},{"type":"function","title":"Beethoven.RoleMgmt.Assign.subscribe/1","doc":"Subscribes to the table mapped to the DistrServer `Elixir.Beethoven.RoleMgmt.Assign`'s mnesia table.\n\n# Matches based on subscription\n\n## `:simple`\n\n    {:mnesia_table_event, {:atom, record(), _op_data}}\n\n## `:detailed`\n\n    {:mnesia_table_event, {:atom, module() | :atom(), record(), [] | [record()], _op_data}}","ref":"Beethoven.RoleMgmt.Assign.html#subscribe/1"},{"type":"function","title":"Beethoven.RoleMgmt.Assign.table_exists?/0","doc":"Checks if the DistrServer `Elixir.Beethoven.RoleMgmt.Assign`'s mnesia table exists.","ref":"Beethoven.RoleMgmt.Assign.html#table_exists?/0"},{"type":"function","title":"Beethoven.RoleMgmt.Assign.until_exists/3","doc":"Holds the thread until the DistrServer `Elixir.Beethoven.RoleMgmt.Assign`'s mnesia table becomes available, or timeout occurs.\nDefaults to `1_000` milliseconds for timeouts and `15` milliseconds for checking intervals.","ref":"Beethoven.RoleMgmt.Assign.html#until_exists/3"},{"type":"module","title":"Beethoven.RoleMgmt.Failover","doc":"Failover monitor for roles.\nListens for updates from `Beethoven.CoreServer` and will reassign work on the `RoleMgmt.Assign.Tracker`.\nWill reassign using `Beethoven.Allocator`.\nThe update to mnesia will be monitored by `Assign` who will handle the assignment to self.","ref":"Beethoven.RoleMgmt.Failover.html"},{"type":"function","title":"Beethoven.RoleMgmt.Failover.alert_me/0","doc":"# CoreServer\n\nTell the local CoreServer that we want to be alerted to changes to cluster node state.\nUpdate will be sent in the form of a cast.","ref":"Beethoven.RoleMgmt.Failover.html#alert_me/0"},{"type":"function","title":"Example - Beethoven.RoleMgmt.Failover.alert_me/0","doc":"{:node_update, {nodeName, status}}","ref":"Beethoven.RoleMgmt.Failover.html#alert_me/0-example"},{"type":"function","title":"Beethoven.RoleMgmt.Failover.check_and_reassign/2","doc":"Removes a given node from a Assign.Tracker record and reassigns the role.","ref":"Beethoven.RoleMgmt.Failover.html#check_and_reassign/2"},{"type":"function","title":"Beethoven.RoleMgmt.Failover.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Beethoven.RoleMgmt.Failover.html#child_spec/1"},{"type":"function","title":"Beethoven.RoleMgmt.Failover.start_link/1","doc":"Supervisor Entry point.","ref":"Beethoven.RoleMgmt.Failover.html#start_link/1"},{"type":"module","title":"Beethoven.RoleMgmt.Manager","doc":"Dynamic supervisor that manages the specialized role PIDs.\n\n# Public API\n- `add_role/3` Adds a role to the Manager Dynamic Supervisor. Stores role name + PID in ETS.\n- `remove_role/1` Removes a hosted role from the Dynamic Supervisor.\n\nThe use of ETS is just for tracking atomic role names to PIDs.\nIt has no awareness of the cluster level allocations for this role.","ref":"Beethoven.RoleMgmt.Manager.html"},{"type":"function","title":"Beethoven.RoleMgmt.Manager.add_role/3","doc":"Adds a role to the `Elixir.Beethoven.RoleMgmt.Manager` Dynamic Supervisor. Adds the pid + roleName to ETS for tracking.\nIf the role already exists, `:ok` will still be returned.","ref":"Beethoven.RoleMgmt.Manager.html#add_role/3"},{"type":"function","title":"Beethoven.RoleMgmt.Manager.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Beethoven.RoleMgmt.Manager.html#child_spec/1"},{"type":"function","title":"Beethoven.RoleMgmt.Manager.remove_role/1","doc":"Removes the role from the `Elixir.Beethoven.RoleMgmt.Manager`Dynamic Supervisor.","ref":"Beethoven.RoleMgmt.Manager.html#remove_role/1"},{"type":"function","title":"Beethoven.RoleMgmt.Manager.start_link/1","doc":"Supervisor Entry point.","ref":"Beethoven.RoleMgmt.Manager.html#start_link/1"},{"type":"module","title":"Beethoven.RoleServer","doc":"# DEPRECATED! Do not use. Use `Beethoven.RoleMgmt` instead.\n\nThis server is responsible for the management and allocation of specialized OTP complaint PIDs.\nRoles are defined in in the application config for `:beethoven`.","ref":"Beethoven.RoleServer.html"},{"type":"module","title":"Example - Beethoven.RoleServer","doc":"config :beethoven,\n      ...\n      roles: [\n        # { ,  ,  ,  }\n        {:test, Beethoven.TestRole, [arg1: \"arg1\"], 1}\n      ]\n\nBased on the definition for each role, the nodes in the `Beethoven` cluster will ensure the required amount of that service is running across the cluster.\nThe number of service instances needed for the role is defined with the `InstanceCount` element in the tuple. These services are spread across the cluster.","ref":"Beethoven.RoleServer.html#module-example"},{"type":"function","title":"Beethoven.RoleServer.alert_me/0","doc":"# CoreServer\n\nTell the local CoreServer that we want to be alerted to changes to cluster node state.\nUpdate will be sent in the form of a cast.","ref":"Beethoven.RoleServer.html#alert_me/0"},{"type":"function","title":"Example - Beethoven.RoleServer.alert_me/0","doc":"{:node_update, {nodeName, status}}","ref":"Beethoven.RoleServer.html#alert_me/0-example"},{"type":"function","title":"Beethoven.RoleServer.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Beethoven.RoleServer.html#child_spec/1"},{"type":"function","title":"Beethoven.RoleServer.copy_tracker/0","doc":"Manually copy the tracking DB to the node.","ref":"Beethoven.RoleServer.html#copy_tracker/0"},{"type":"function","title":"Beethoven.RoleServer.dirty_select/1","doc":"Similar to `:mnesia.dirty_select/2` but only needs the match spec as an argument.\nThe table name of the DistrServer `Elixir.Beethoven.RoleServer`'s mnesia table is input automatically as the 1st arity.","ref":"Beethoven.RoleServer.html#dirty_select/1"},{"type":"function","title":"Beethoven.RoleServer.fetch/1","doc":"Fetches data from the DistrServer `Elixir.Beethoven.RoleServer`'s mnesia table.\nUses a record key to query the data. Will return all matching records.","ref":"Beethoven.RoleServer.html#fetch/1"},{"type":"function","title":"Beethoven.RoleServer.fetch_all/0","doc":"Fetches all records from the DistrServer `Elixir.Beethoven.RoleServer`'s mnesia table.","ref":"Beethoven.RoleServer.html#fetch_all/0"},{"type":"function","title":"Beethoven.RoleServer.get_role_config/0","doc":"Retrieves roles from config and converts to map.","ref":"Beethoven.RoleServer.html#get_role_config/0"},{"type":"function","title":"Beethoven.RoleServer.get_table_name/0","doc":"Returns the name of the DistrServer `Elixir.Beethoven.RoleServer`'s mnesia table.","ref":"Beethoven.RoleServer.html#get_table_name/0"},{"type":"function","title":"Beethoven.RoleServer.role_list_to_map/1","doc":"Creates a map from a list of maps. First element of the map needs to be an atom.\nThis same atom will be the key for the rest of the data in the map.","ref":"Beethoven.RoleServer.html#role_list_to_map/1"},{"type":"type","title":"Beethoven.RoleServer.roleMap/0","doc":"Role list in the form of a map.","ref":"Beethoven.RoleServer.html#t:roleMap/0"},{"type":"type","title":"Beethoven.RoleServer.roleRecord/0","doc":"Simplified type for tracker records.\nJust excludes the table name from the record tuple.","ref":"Beethoven.RoleServer.html#t:roleRecord/0"},{"type":"function","title":"Beethoven.RoleServer.start_assign/0","doc":"Starts assignment job on the RoleServer.","ref":"Beethoven.RoleServer.html#start_assign/0"},{"type":"function","title":"Beethoven.RoleServer.start_link/1","doc":"Supervisor Entry point.","ref":"Beethoven.RoleServer.html#start_link/1"},{"type":"function","title":"Beethoven.RoleServer.subscribe/1","doc":"Subscribes to the table mapped to the DistrServer `Elixir.Beethoven.RoleServer`'s mnesia table.\n\n# Matches based on subscription\n\n## `:simple`\n\n    {:mnesia_table_event, {:atom, record(), _op_data}}\n\n## `:detailed`\n\n    {:mnesia_table_event, {:atom, module() | :atom(), record(), [] | [record()], _op_data}}","ref":"Beethoven.RoleServer.html#subscribe/1"},{"type":"function","title":"Beethoven.RoleServer.table_exists?/0","doc":"Checks if the DistrServer `Elixir.Beethoven.RoleServer`'s mnesia table exists.","ref":"Beethoven.RoleServer.html#table_exists?/0"},{"type":"function","title":"Beethoven.RoleServer.until_exists/3","doc":"Holds the thread until the DistrServer `Elixir.Beethoven.RoleServer`'s mnesia table becomes available, or timeout occurs.\nDefaults to `1_000` milliseconds for timeouts and `15` milliseconds for checking intervals.","ref":"Beethoven.RoleServer.html#until_exists/3"},{"type":"module","title":"Beethoven.SeekChat","doc":"Module to simplify the interface between nodes during the locator Seeking flow.","ref":"Beethoven.SeekChat.html"},{"type":"function","title":"Beethoven.SeekChat.decode/1","doc":"Deserializes a binary into `msg()` type.","ref":"Beethoven.SeekChat.html#decode/1"},{"type":"function","title":"Beethoven.SeekChat.encode/1","doc":"Serializes a `msg()` object into a binary.","ref":"Beethoven.SeekChat.html#encode/1"},{"type":"type","title":"Beethoven.SeekChat.msg/0","doc":"Type for sending messages between Locator and Listening Servers.","ref":"Beethoven.SeekChat.html#t:msg/0"},{"type":"function","title":"Beethoven.SeekChat.new_msg/2","doc":"Creates a new msg, in the form of `msg()`.\n# `type()`\n- `:seeking` --> (CLIENT) Client looking to join.\n- `:watching` --> (CLIENT) Client checking if anything else is listening.\n- `:reply` --> (SERVER) Server response to either message\n---\n# `payload()`:\n- `:join` --> (CLIENT) Seeking to join a cluster.\n- `:check` --> (CLIENT) Client is watching, and wants to know if we are in a cluster.\n- `:self` --> (SERVER) When client calls its own listener.\n- `list(node())` --> (SERVER) When client is watching, and server is in a cluster.\n- `:standalone` --> (SERVER) When client is watching, and the server is in standalone mode.\n- `:joined` --> (SERVER) Client request to join was successful.\n- `:join_failed` --> (SERVER) Client request to join failed.\n- `:error` --> (SERVER) Generic error occurred.","ref":"Beethoven.SeekChat.html#new_msg/2"},{"type":"type","title":"Beethoven.SeekChat.payload/0","doc":"Message payload.\n# Options:\n- `:join` --> (CLIENT) Seeking to join a cluster.\n- `:check` --> (CLIENT) Client is watching, and wants to know if we are in a cluster.\n- `:self` --> (SERVER) When client calls its own listener.\n- `list(node())` --> (SERVER) When client is watching, and server is in a cluster.\n- `:standalone` --> (SERVER) When client is watching, and the server is in standalone mode.\n- `:joined` --> (SERVER) Client request to join was successful.\n- `:join_failed` --> (SERVER) Client request to join failed.\n- `:error` --> (SERVER) Generic error occurred.","ref":"Beethoven.SeekChat.html#t:payload/0"},{"type":"type","title":"Beethoven.SeekChat.type/0","doc":"Request type for a node seeking message.\n# Client Options\n- `:seeking` --> Client looking to join.\n- `:watching` --> Client checking if anything else is listening.\n# Server Options\n- `:reply` --> Server response to either message","ref":"Beethoven.SeekChat.html#t:type/0"},{"type":"module","title":"Beethoven.Signals","doc":"Beethoven Allocator signals for the Beethoven service itself.","ref":"Beethoven.Signals.html"},{"type":"function","title":"Beethoven.Signals.decrement_beethoven_role_count/0","doc":"","ref":"Beethoven.Signals.html#decrement_beethoven_role_count/0"},{"type":"function","title":"Beethoven.Signals.increment_beethoven_role_count/0","doc":"","ref":"Beethoven.Signals.html#increment_beethoven_role_count/0"},{"type":"module","title":"Beethoven.Substrate","doc":"Supervisor for core services.\nThese are the services that need to start once Locator has determined the cluster state.\n\n# Nested services:\n- `Beethoven.Allocator` Stack of services to track and aggregate signals to determine how busy a node is.\n- `Beethoven.HwMon` Hardware monitor stack. Tracks hardware resource consumption and both stores the data historically and sends signals to `Allocator`.\n- `Beethoven.CoreServer` Node monitoring service. Can send signals to callers when node state changes.\n- `Beethoven.BeaconServer` TCP server for Beethoven instances to find each other. This interacts with the `Locator` service on other nodes.\n- `Beethoven.RoleMgmt` Role management and allocation service that can host roles defined in config.ex.","ref":"Beethoven.Substrate.html"},{"type":"function","title":"Beethoven.Substrate.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Beethoven.Substrate.html#child_spec/1"},{"type":"function","title":"Beethoven.Substrate.start_link/1","doc":"Supervisor Entry point.","ref":"Beethoven.Substrate.html#start_link/1"},{"type":"module","title":"Beethoven.TestRole","doc":"Test role to test RoleServer operations.\n\nA role is just another PID of some kind. Module must have `start_link/1` that returns {:ok, pid()} or the role will fail to load.","ref":"Beethoven.TestRole.html"},{"type":"function","title":"Beethoven.TestRole.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Beethoven.TestRole.html#child_spec/1"},{"type":"function","title":"Beethoven.TestRole.start_link/1","doc":"Entry point for RolServer/Supervisor","ref":"Beethoven.TestRole.html#start_link/1"},{"type":"module","title":"Beethoven.TestdRole","doc":"Example role that leverages DistrServer instead of GenServer.","ref":"Beethoven.TestdRole.html"},{"type":"function","title":"Beethoven.TestdRole.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Beethoven.TestdRole.html#child_spec/1"},{"type":"function","title":"Beethoven.TestdRole.dirty_select/1","doc":"Similar to `:mnesia.dirty_select/2` but only needs the match spec as an argument.\nThe table name of the DistrServer `Elixir.Beethoven.TestdRole`'s mnesia table is input automatically as the 1st arity.","ref":"Beethoven.TestdRole.html#dirty_select/1"},{"type":"function","title":"Beethoven.TestdRole.fetch/1","doc":"Fetches data from the DistrServer `Elixir.Beethoven.TestdRole`'s mnesia table.\nUses a record key to query the data. Will return all matching records.","ref":"Beethoven.TestdRole.html#fetch/1"},{"type":"function","title":"Beethoven.TestdRole.fetch_all/0","doc":"Fetches all records from the DistrServer `Elixir.Beethoven.TestdRole`'s mnesia table.","ref":"Beethoven.TestdRole.html#fetch_all/0"},{"type":"function","title":"Beethoven.TestdRole.get_table_name/0","doc":"Returns the name of the DistrServer `Elixir.Beethoven.TestdRole`'s mnesia table.","ref":"Beethoven.TestdRole.html#get_table_name/0"},{"type":"function","title":"Beethoven.TestdRole.start_link/1","doc":"","ref":"Beethoven.TestdRole.html#start_link/1"},{"type":"function","title":"Beethoven.TestdRole.subscribe/1","doc":"Subscribes to the table mapped to the DistrServer `Elixir.Beethoven.TestdRole`'s mnesia table.\n\n# Matches based on subscription\n\n## `:simple`\n\n    {:mnesia_table_event, {:atom, record(), _op_data}}\n\n## `:detailed`\n\n    {:mnesia_table_event, {:atom, module() | :atom(), record(), [] | [record()], _op_data}}","ref":"Beethoven.TestdRole.html#subscribe/1"},{"type":"function","title":"Beethoven.TestdRole.table_exists?/0","doc":"Checks if the DistrServer `Elixir.Beethoven.TestdRole`'s mnesia table exists.","ref":"Beethoven.TestdRole.html#table_exists?/0"},{"type":"function","title":"Beethoven.TestdRole.until_exists/3","doc":"Holds the thread until the DistrServer `Elixir.Beethoven.TestdRole`'s mnesia table becomes available, or timeout occurs.\nDefaults to `1_000` milliseconds for timeouts and `15` milliseconds for checking intervals.","ref":"Beethoven.TestdRole.html#until_exists/3"},{"type":"module","title":"Beethoven.Utils","doc":"Module for generic utilities.","ref":"Beethoven.Utils.html"},{"type":"function","title":"Beethoven.Utils.get_app_env/2","doc":"Gets an environmental variable from beethoven's application config","ref":"Beethoven.Utils.html#get_app_env/2"},{"type":"function","title":"Beethoven.Utils.monitor_all_nodes/1","doc":"Toggles the monitoring status of **all** nodes in the cluster.","ref":"Beethoven.Utils.html#monitor_all_nodes/1"},{"type":"function","title":"Beethoven.Utils.monitor_node/2","doc":"Toggles the monitoring status of another node in the cluster.","ref":"Beethoven.Utils.html#monitor_node/2"},{"type":"function","title":"Beethoven.Utils.random_backoff/1","doc":"Performs a backoff based on a random number within a provided range.\nTime will be used as milliseconds. Returns time waited with `:ok` atom.","ref":"Beethoven.Utils.html#random_backoff/1"},{"type":"function","title":"Beethoven.Utils.remote_call/3","doc":"Fn to streamline execution of functions on other nodes in the cluster.","ref":"Beethoven.Utils.html#remote_call/3"},{"type":"module","title":"Templates.DistrServer","doc":"","ref":"Templates.DistrServer.html"},{"type":"function","title":"Templates.DistrServer.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Templates.DistrServer.html#child_spec/1"},{"type":"function","title":"Templates.DistrServer.dirty_select/1","doc":"Similar to `:mnesia.dirty_select/2` but only needs the match spec as an argument.\nThe table name of the DistrServer `Elixir.Templates.DistrServer`'s mnesia table is input automatically as the 1st arity.","ref":"Templates.DistrServer.html#dirty_select/1"},{"type":"function","title":"Templates.DistrServer.fetch/1","doc":"Fetches data from the DistrServer `Elixir.Templates.DistrServer`'s mnesia table.\nUses a record key to query the data. Will return all matching records.","ref":"Templates.DistrServer.html#fetch/1"},{"type":"function","title":"Templates.DistrServer.fetch_all/0","doc":"Fetches all records from the DistrServer `Elixir.Templates.DistrServer`'s mnesia table.","ref":"Templates.DistrServer.html#fetch_all/0"},{"type":"function","title":"Templates.DistrServer.get_table_name/0","doc":"Returns the name of the DistrServer `Elixir.Templates.DistrServer`'s mnesia table.","ref":"Templates.DistrServer.html#get_table_name/0"},{"type":"function","title":"Templates.DistrServer.start_link/1","doc":"Supervisor Entry point.","ref":"Templates.DistrServer.html#start_link/1"},{"type":"function","title":"Templates.DistrServer.subscribe/1","doc":"Subscribes to the table mapped to the DistrServer `Elixir.Templates.DistrServer`'s mnesia table.\n\n# Matches based on subscription\n\n## `:simple`\n\n    {:mnesia_table_event, {:atom, record(), _op_data}}\n\n## `:detailed`\n\n    {:mnesia_table_event, {:atom, module() | :atom(), record(), [] | [record()], _op_data}}","ref":"Templates.DistrServer.html#subscribe/1"},{"type":"function","title":"Templates.DistrServer.table_exists?/0","doc":"Checks if the DistrServer `Elixir.Templates.DistrServer`'s mnesia table exists.","ref":"Templates.DistrServer.html#table_exists?/0"},{"type":"function","title":"Templates.DistrServer.until_exists/3","doc":"Holds the thread until the DistrServer `Elixir.Templates.DistrServer`'s mnesia table becomes available, or timeout occurs.\nDefaults to `1_000` milliseconds for timeouts and `15` milliseconds for checking intervals.","ref":"Templates.DistrServer.html#until_exists/3"},{"type":"module","title":"Templates.GenServer","doc":"","ref":"Templates.GenServer.html"},{"type":"function","title":"Templates.GenServer.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Templates.GenServer.html#child_spec/1"},{"type":"function","title":"Templates.GenServer.start_link/1","doc":"Supervisor Entry point.","ref":"Templates.GenServer.html#start_link/1"},{"type":"module","title":"Templates.GenServerEts","doc":"","ref":"Templates.GenServerEts.html"},{"type":"function","title":"Templates.GenServerEts.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Templates.GenServerEts.html#child_spec/1"},{"type":"function","title":"Templates.GenServerEts.start_link/1","doc":"Supervisor Entry point.","ref":"Templates.GenServerEts.html#start_link/1"},{"type":"module","title":"Templates.Supervisor","doc":"","ref":"Templates.Supervisor.html"},{"type":"function","title":"Templates.Supervisor.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Templates.Supervisor.html#child_spec/1"},{"type":"function","title":"Templates.Supervisor.start_link/1","doc":"Supervisor Entry point.","ref":"Templates.Supervisor.html#start_link/1"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.37.3"}}